@@ -1,5 +1,5 @@
 /*
-*   $Id: c.c,v 1.24 2003/09/03 02:53:46 darren Exp $
+*   $Id: c.c,v 1.28 2004/02/28 15:37:52 darren Exp $
 *
 *   Copyright (c) 1996-2003, Darren Hiebert
 *
@@ -565,7 +565,7 @@ static const char *implementationString (const impType imp)
 static const char *tokenString (const tokenType type)
 {
     static const char *const names [] = {
-	"none", "args", "}", "{", "comma", "colon", "double colon", "keyword",
+	"none", "args", "}", "{", "colon", "comma", "double colon", "keyword",
 	"name", "package", "paren-name", "semicolon", "specifier"
     };
     Assert (sizeof (names) / sizeof (names [0]) == TOKEN_COUNT);
@@ -1228,8 +1228,6 @@ static void qualifyVariableTag (const statementInfo *const st,
      */
     if (! isType (nameToken, TOKEN_NAME))
 	;
-    else if (st->declaration == DECL_IGNORE)
-	;
     else if (st->scope == SCOPE_TYPEDEF)
 	makeTag (nameToken, st, TRUE, TAG_TYPEDEF);
     else if (st->declaration == DECL_EVENT)
@@ -1646,6 +1644,12 @@ static void readParents (statementInfo *const st, const int qualifier)
     deleteToken (token);
 }
 
+static void skipStatement (statementInfo *const st)
+{
+    st->declaration = DECL_IGNORE;
+    skipToOneOf (";");
+}
+
 static void processToken (tokenInfo *const token, statementInfo *const st)
 {
     switch (token->keyword)		/* is it a reserved word? */
@@ -1668,10 +1672,10 @@ static void processToken (tokenInfo *const token, statementInfo *const st)
 	case KEYWORD_FLOAT:	st->declaration = DECL_BASE;		break;
 	case KEYWORD_FUNCTION:	st->declaration = DECL_BASE;		break;
 	case KEYWORD_FRIEND:	st->scope	= SCOPE_FRIEND;		break;
-	case KEYWORD_GOTO:	st->declaration = DECL_IGNORE;		break;
+	case KEYWORD_GOTO:	skipStatement (st);			break;
 	case KEYWORD_IMPLEMENTS:readParents (st, '.');
 				setToken (st, TOKEN_NONE);		break;
-	case KEYWORD_IMPORT:	st->declaration = DECL_IGNORE;		break;
+	case KEYWORD_IMPORT:	skipStatement (st);			break;
 	case KEYWORD_INT:	st->declaration = DECL_BASE;		break;
 	case KEYWORD_INTEGER:	st->declaration = DECL_BASE;		break;
 	case KEYWORD_INTERFACE: st->declaration = DECL_INTERFACE;	break;
@@ -1684,7 +1688,7 @@ static void processToken (tokenInfo *const token, statementInfo *const st)
 	case KEYWORD_PROGRAM:	st->declaration = DECL_PROGRAM;		break;
 	case KEYWORD_PROTECTED:	setAccess (st, ACCESS_PROTECTED);	break;
 	case KEYWORD_PUBLIC:	setAccess (st, ACCESS_PUBLIC);		break;
-	case KEYWORD_RETURN:	st->declaration = DECL_IGNORE;		break;
+	case KEYWORD_RETURN:	skipStatement (st);			break;
 	case KEYWORD_SHORT:	st->declaration = DECL_BASE;		break;
 	case KEYWORD_SIGNED:	st->declaration = DECL_BASE;		break;
 	case KEYWORD_STRING:	st->declaration = DECL_BASE;		break;
@@ -1693,7 +1697,7 @@ static void processToken (tokenInfo *const token, statementInfo *const st)
 	case KEYWORD_THROWS:	discardTypeList (token);		break;
 	case KEYWORD_UNION:	st->declaration = DECL_UNION;		break;
 	case KEYWORD_UNSIGNED:	st->declaration = DECL_BASE;		break;
-	case KEYWORD_USING:	st->declaration = DECL_IGNORE;		break;
+	case KEYWORD_USING:	skipStatement (st);			break;
 	case KEYWORD_VOID:	st->declaration = DECL_BASE;		break;
 	case KEYWORD_VOLATILE:	st->declaration = DECL_BASE;		break;
 	case KEYWORD_VIRTUAL:	st->implementation = IMP_VIRTUAL;	break;
@@ -2042,13 +2046,11 @@ static int parseParens (statementInfo *const st, parenInfo *const info)
 
 	    case '[':
 		info->isKnrParamList = FALSE;
-		vStringPut (Signature, c);
 		skipToMatch ("[]");
 		break;
 
 	    case '<':
 		info->isKnrParamList = FALSE;
-		vStringPut (Signature, c);
 		skipToMatch ("<>");
 		break;
 
@@ -2251,7 +2253,8 @@ static void processColon (statementInfo *const st)
 	    const tokenInfo *const prev  = prevToken (st, 1);
 	    const tokenInfo *const prev2 = prevToken (st, 2);
 	    if (prev->keyword == KEYWORD_DEFAULT ||
-		prev2->keyword == KEYWORD_CASE)
+		prev2->keyword == KEYWORD_CASE ||
+		st->parent != NULL)
 	    {
 		reinitStatement (st, FALSE);
 	    }
@@ -2259,6 +2262,17 @@ static void processColon (statementInfo *const st)
     }
 }
 
+static void processAngleBracket (void)
+{
+    int c = cppGetc ();
+    if (c == '>')
+	;   /* already found match for template */
+    else if (isLanguage (Lang_cpp) &&  c != '<' && c != '=')
+	skipToMatch ("<>");        /* this is a template */
+    else
+	cppUngetc (c);
+}
+
 /*  Skips over any initializing value which may follow an '=' character in a
  *  variable definition.
  */
@@ -2361,12 +2375,6 @@ static void parseGeneralToken (statementInfo *const st, const int c)
 		cppUngetc (c2);
 	}
     }
-    else if (c == '<')
-    {
-	int c2 = cppGetc ();
-	if (st->declaration == DECL_CLASS && !(c2 == '=' || isspace (c2)))
-	    skipToMatch ("<>");        /* this is a template */
-    }
     else if (c == '!' || c == '>')
     {
 	int c2 = cppGetc ();
@@ -2393,6 +2401,7 @@ static void nextToken (statementInfo *const st)
 	{
 	    case EOF: longjmp (Exception, (int) ExceptionEOF);	break;
 	    case '(': analyzeParens (st);			break;
+	    case '<': processAngleBracket ();			break;
 	    case '*': st->haveQualifyingName = FALSE;		break;
 	    case ',': setToken (st, TOKEN_COMMA);		break;
 	    case ':': processColon (st);			break;
@@ -2439,7 +2448,7 @@ static void deleteStatement (void)
 
     for (i = 0  ;  i < (unsigned int) NumTokens  ;  ++i)
     {
-	deleteToken (st->token [i]);        st->token [i] = NULL;
+	deleteToken (st->token [i]);       st->token [i] = NULL;
     }
     deleteToken (st->blockName);           st->blockName = NULL;
     deleteToken (st->context);             st->context = NULL;
