@@ -1,7 +1,8 @@
 /*
-*   $Id: ruby.c,v 1.2 2001/12/18 04:30:18 darren Exp $
+*   $Id: ruby.c,v 1.4 2002/09/05 00:35:37 darren Exp $
 *
 *   Copyright (c) 2000-2001, Thaddeus Covert <sahuagin@mediaone.net>
+*   Copyright (c) 2002 Matthias Veit <matthias_veit@yahoo.de>
 *
 *   This source code is released for free distribution under the terms of the
 *   GNU General Public License.
@@ -25,13 +26,14 @@
 *   DATA DEFINITIONS
 */
 typedef enum {
-    K_CLASS, K_FUNCTION
+    K_CLASS, K_METHOD, K_SINGLETON, K_MIXIN
 } rubyKind;
 
 static kindOption RubyKinds [] = {
-    { TRUE, 'c', "class",    "classes" },
-    { TRUE, 'f', "function", "functions" },
-    { TRUE, 'm', "mixin",    "mixins" }
+    { TRUE, 'c', "class",  "classes" },
+    { TRUE, 'f', "method", "methods" },
+    { TRUE, 'F', "singleton method", "singleton methods" },
+    { TRUE, 'm', "mixin",  "mixins" }
 };
 
 
@@ -47,32 +49,49 @@ static void findRubyTags (void)
 
     while ((line = fileReadLine ()) != NULL)
     {
-	const unsigned char *cp = line;
+        const unsigned char *cp = line;
+        boolean is_singleton = FALSE;
 
-	while (*cp != '\0')
+        while (*cp != '\0')
 	{
-	    if (*cp=='"' &&
-		strncmp ((const char*) cp, "\"\"\"", (size_t) 3) == 0)
+	    if (strncmp ((const char*) cp, "\"\"\"", (size_t) 3) == 0)
 	    {
-		inMultilineString = (boolean) !inMultilineString;
 		cp += 3;
+		inMultilineString = (boolean) !inMultilineString;
 	    }
 	    /* make sure you include the comments */
-	    if(*cp=='=' && strncmp((const char*)cp, "==begin", (size_t)7) == 0)
-	      {
-		inMultilineString = (boolean)!inMultilineString;
-		cp +=3;
-	      }
+	    if (strncmp ((const char*) cp, "==begin", (size_t) 7) == 0)
+	    {
+		cp += 7;
+		inMultilineString = (boolean) !inMultilineString;
+	    }
 	    /* mark the end of a comment */
-	    if( *cp=='=' && strncmp((const char*)cp, "==end", (size_t)5) == 0)
-	      {
-		inMultilineString = (boolean)0;
-		cp+=5;
-	      }
+	    if (strncmp ((const char*) cp, "==end", (size_t) 5) == 0)
+	    {
+		inMultilineString = FALSE;
+		cp += 5;
+	    }
 	    if (inMultilineString  ||  isspace ((int) *cp))
 		++cp;
 	    else if (*cp == '#')
 		break;
+	    else if (strncmp ((const char*) cp, "module", (size_t) 6) == 0)
+	    {
+		cp += 6;
+		if (isspace ((int) *cp))
+		{
+		    while (isspace ((int) *cp))
+			++cp;
+		    while (isalnum ((int) *cp)  ||  *cp == '_')
+		    {
+			vStringPut (name, (int) *cp);
+			++cp;
+		    }
+		    vStringTerminate (name);
+		    makeSimpleTag (name, RubyKinds, K_MIXIN);
+		    vStringClear (name);
+		}
+	    }
 	    else if (strncmp ((const char*) cp, "class", (size_t) 5) == 0)
 	    {
 		cp += 5;
@@ -99,20 +118,37 @@ static void findRubyTags (void)
 			++cp;
 
 		    /* Put the valid characters allowed in a variable name
-		     * in here. In ruby a variable name ENDING in ! means
-		     * it changes the underlying data structure in place.
-		     * A variable name ENDING in ? means that the function
-		     * returns a bool. Functions should not start with these
-		     * characters.
-		     */
-		    while (isalnum ((int) *cp)  ||  *cp == '_' ||
-			   *cp == '!' || *cp =='?')
+			* in here. In ruby a variable name ENDING in ! means
+			* it changes the underlying data structure in place.
+			* A variable name ENDING in ? means that the function
+			* returns a bool. Functions should not start with these
+			* characters.
+			*/
+		    while (isalnum ((int) *cp)  ||
+			((int) *cp != '\0' &&
+			 strchr ("_!?.", (int) *cp) != NULL))
 		    {
-			vStringPut (name, (int) *cp);
+			/* classmethods are accesible only via class instance
+			    * instead of object instance. This difference has to
+			    * be outlined.
+			    */
+			if (*cp == '.')
+			{
+			    is_singleton = TRUE;
+			    vStringTerminate (name);
+			    vStringClear(name);
+			}
+			else
+			{
+			    vStringPut (name, (int) *cp);
+			}
 			++cp;
 		    }
 		    vStringTerminate (name);
-		    makeSimpleTag (name, RubyKinds, K_FUNCTION);
+		    if (is_singleton)
+			makeSimpleTag (name, RubyKinds, K_SINGLETON);
+		    else
+			makeSimpleTag (name, RubyKinds, K_METHOD);
 		    vStringClear (name);
 		}
 	    }
@@ -135,5 +171,5 @@ extern parserDefinition* RubyParser (void)
     def->kindCount  = KIND_COUNT (RubyKinds);
     def->extensions = extensions;
     def->parser     = findRubyTags;
-    return def;  
+    return def;
 }
